@@ -11,7 +11,9 @@ __all__ = (
     'video2np',
     'np2video',
     'Run',
-    'run')
+    'run',
+    'sound_chunks',
+    'video_frames')
 
 try:
     import numm.opencv as opencv
