@@ -14,8 +14,6 @@ import threading
 import time
 
 import gobject
-import pygst
-pygst.require("0.10")
 import gst
 import numpy
 
