@@ -6,11 +6,6 @@ Audio data is represented by arrays of shape (n_frames, n_channels),
 with a sample rate of 44.1kHz by default.
 """
 
-import Queue
-
-import pygst
-pygst.require('0.10')
-
 import gobject
 
 import gst
@@ -40,11 +35,10 @@ class SoundReader(numm.io.Reader):
     def __init__(self, path, cb, sample_rate=44100, start=0, n_frames=-1):
         # XXX: These caps ought to be kept in sync with the ones in numm.run.
         (pipeline, appsink) = _make_sound_pipeline(path, sample_rate)
-        numm.io.Reader.__init__(self, pipeline, appsink)
+        numm.io.Reader.__init__(self, pipeline, appsink, cb)
 
-        self.cb = cb
         self.sample_rate = sample_rate
-        self.start = start
+        self.start_t = start
         self.n_frames = n_frames
 
         self.frame_idx = 0
@@ -55,17 +49,14 @@ class SoundReader(numm.io.Reader):
         # XXX: Check return value?
         self.pipeline.seek_simple(
             gst.FORMAT_TIME, gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE,
-            int(self.start * gst.SECOND / float(self.sample_rate)))
+            int(self.start_t * gst.SECOND / float(self.sample_rate)))
 
-    def _new_buffer(self, _appsink):
-        numm.io.Reader._new_buffer(self, _appsink)
-        buffer = self.pull()
-
-        if self.start != 0 and not self.seek_done:
+    def _process_buffer(self, buffer):
+        if self.start_t != 0 and not self.seek_done:
             self.seek_done = True
             #print 'seeking to %d' % (int(start*gst.SECOND/float(sample_rate)))
             gobject.idle_add(self._do_seek)
-            return True
+            return None
 
         if self.n_channels is None:
             self.n_channels = buffer.caps[0]['channels']
@@ -75,44 +66,25 @@ class SoundReader(numm.io.Reader):
         np.timestamp = buffer.timestamp
 
         if self.n_frames > 0 and self.frame_idx + np.shape[0] >= self.n_frames:
-            self.cb(np[:self.n_frames - self.frame_idx])
-            return False
+            np = np[:self.n_frames - self.frame_idx]
+
+        if np.shape[0] == 0:
+            self.eos = True
+            return None
 
         self.frame_idx += np.shape[0]
-        self.cb(np)
-        return True
-
-    # numm.io._run_appsink_pipeline(pipeline, appsink, loaded_buffer)
-
-def _read_sound(path, cb, **kw):
-    loader = SoundReader(path, cb, **kw)
-    r = numm.io.RunPipeline(loader.pipeline)
-
-    try:
-        r.start()
-
-        while not r.done:
-            r.step()
-    finally:
-        r.stop()
-
-    if r.errors:
-        raise r.errors[0]
+        return np
 
 def sound_chunks(path, **kw):
-    def cb(a):
-        q.put(a)
+    chunks = []
+    reader = SoundReader(path, chunks.append, **kw)
 
-    loader = SoundReader(path, cb, **kw)
-    r = numm.io.RunPipeline(loader.pipeline)
-    q = Queue.Queue(1)
+    for _ in reader:
+        while chunks:
+            yield chunks.pop(0)
 
-    for _ in r.run():
-        while q.qsize() > 0:
-            yield q.get()
-
-    while q.qsize() > 0:
-        yield q.get()
+    while chunks:
+        yield chunks.pop(0)
 
 def _write_sound(np, filepath, sample_rate=44100):
     pipeline = gst.parse_launch(
@@ -138,15 +110,12 @@ def sound2np(filepath, start=0, n_frames=-1, sample_rate=44100):
     Load audio data from a file.
     """
 
-    arrays = []
+    chunks = []
+    reader = SoundReader(filepath, chunks.append, start=start, n_frames=n_frames, sample_rate=sample_rate)
+    reader.run()
 
-    def cb(a):
-        arrays.append(a)
-
-    _read_sound(filepath, cb, start=start, n_frames=n_frames, sample_rate=sample_rate)
-
-    if arrays:
-        return numpy.concatenate(arrays)
+    if chunks:
+        return numpy.concatenate(chunks)
     else:
         return numpy.ndarray((0, 2))
 
