@@ -4,11 +4,6 @@
 
 from __future__ import division
 
-import Queue
-
-import pygst
-pygst.require('0.10')
-
 import gst
 import gobject
 import numpy
@@ -51,9 +46,8 @@ def _calculate_size(caps, height):
 
     return (width, height)
 
-def _make_video_pipeline(path, width, height, framerate):
+def _make_video_pipeline(src, width, height, framerate):
     pipeline = gst.parse_launch('''
-        filesrc name=filesrc !
         decodebin2 name=decodebin !
         videorate !
         videoscale !
@@ -62,13 +56,13 @@ def _make_video_pipeline(path, width, height, framerate):
         appsink name=sink sync=false
         ''')
 
-    src = pipeline.get_by_name('filesrc')
-    src.props.location = path
-
     sbin = pipeline.get_by_name('decodebin')
     sbin.props.caps = gst.Caps("video/x-raw-yuv;video/x-raw-rgb")
     sbin.props.expose_all_streams = False
 
+    pipeline.add(src)
+    src.link(sbin)
+
     def pad_caps_changed(pad, pspec):
         pad_caps = pad.get_caps()
 
@@ -125,16 +119,21 @@ def _make_video_pipeline(path, width, height, framerate):
     return (pipeline, sink)
 
 class VideoReader(numm.io.Reader):
-    def __init__(self, path, cb, width=None, height=96, fps=30, start=0,
+    def __init__(self, src, cb, width=None, height=96, fps=30, start=0,
                  n_frames=-1):
+        if isinstance(src, basestring):
+            src_ = gst.element_factory_make('filesrc')
+            src_.props.location = src
+        else:
+            src_ = src
+
         framerate = gst.Fraction(fps, 1)
         (pipeline, appsink) = _make_video_pipeline(
-            path, width, height, framerate)
-        numm.io.Reader.__init__(self, pipeline, appsink)
+            src_, width, height, framerate)
+        numm.io.Reader.__init__(self, pipeline, appsink, cb)
 
-        self.cb = cb
         self.fps = fps
-        self.start = start
+        self.start_t = start
         self.n_frames = n_frames
 
         self.next_frame = 0
@@ -146,26 +145,24 @@ class VideoReader(numm.io.Reader):
         # XXX: Check return value?
         self.pipeline.seek_simple(
             gst.FORMAT_TIME, gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE,
-            int(self.start * gst.SECOND / float(self.fps)))
+            int(self.start_t * gst.SECOND / float(self.fps)))
 
-    def _new_buffer(self, _appsink):
-        numm.io.Reader._new_buffer(self, _appsink)
-        buffer = self.pull()
-
-        if self.start != 0 and not self.seek_done:
+    def _process_buffer(self, buffer):
+        if self.start_t != 0 and not self.seek_done:
             self.seek_done = True
             #print 'seeking to %d' % (int(start*gst.SECOND/float(fps)))
             gobject.idle_add(self._do_seek)
-            return
+            return None
 
         if self.n_frames > 0 and self.next_frame >= self.n_frames:
             # We've already gotten as many frames as we want; stop.
-            return
+            self.eos = True
+            return None
 
         if buffer.timestamp <= self.last_timestamp:
             # This looks like a duplicate buffer. This seems to happen
             # when we seek sometimes.
-            return True
+            return None
 
         if self.shape is None:
             caps = buffer.caps
@@ -178,34 +175,23 @@ class VideoReader(numm.io.Reader):
             self.shape = (h, w, 3)
 
         a = numpy.fromstring(buffer, dtype=numpy.uint8).reshape(self.shape).view(NummBuffer)
+        assert a is not None
         a.timestamp = buffer.timestamp
 
-        self.cb(a)
         self.next_frame += 1
         self.last_timestamp = buffer.timestamp
-
-def _read_video(path, cb, **kw):
-    loader = VideoReader(path, cb, **kw)
-    r = numm.io.RunPipeline(loader.pipeline)
-
-    for _ in r.run():
-        pass
+        return a
 
 def video_frames(path, **kw):
-    q = Queue.Queue(1)
+    frames = []
+    reader = VideoReader(path, frames.append, **kw)
 
-    def cb(frame):
-        q.put(frame)
+    for _ in reader:
+        while frames:
+            yield frames.pop(0)
 
-    loader = VideoReader(path, cb, **kw)
-    r = numm.io.RunPipeline(loader.pipeline)
-
-    for _ in r.run():
-        while q.qsize() > 0:
-            yield q.get()
-
-    while q.qsize() > 0:
-        yield q.get()
+    while frames:
+        yield frames.pop(0)
 
 def _write_video(np, filepath, opts={}):
     defaults = {
@@ -245,11 +231,9 @@ def video2np(path, width=None, height=96, fps=30, start=0, n_frames=-1):
     # output array.
 
     frames = []
-
-    def loaded_frame(frame):
-        frames.append(frame)
-
-    _read_video(path, loaded_frame, width=width, height=height, fps=fps, start=start, n_frames=n_frames)
+    reader = VideoReader(
+        path, frames.append, width=width, height=height, fps=fps, start=start, n_frames=n_frames)
+    reader.run()
 
     if frames:
         (height, width) = frames[0].shape[:2]
